from ribosome.test.integration import (IntegrationSpec, VimIntegrationSpec,
                                     PluginIntegrationSpec)

__all__ = ('VimIntegrationSpec', 'IntegrationSpec', 'PluginIntegrationSpec')
